// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#include "krdpserverdata.h"

#include "krdpserversettings.h"




KRDPServerData::KRDPServerData(QObject *parent)
: KCModuleData(parent)
, m_settings(new KRDPServerSettings(this))

{
autoRegisterSkeletons();
}

KRDPServerSettings *KRDPServerData::settings() const
{
    return m_settings;
}





#include "moc_krdpserverdata.cpp"
