// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#ifndef KRDPSERVERDATA_H
#define KRDPSERVERDATA_H

#include <QObject>

#include "kcmoduledata.h"



class KRDPServerSettings;


class KRDPServerData : public KCModuleData
{
    Q_OBJECT

public:
    explicit KRDPServerData(QObject *parent = nullptr);
    KRDPServerSettings *settings() const;


private:
    KRDPServerSettings *m_settings;

};



#endif
